from indicoio.utils.api import api_handler

def keywords(text, cloud=None, batch=False, api_key=None, version=None, **kwargs):
    """
    Given input text, returns series of keywords and associated scores

    Example usage:

    .. code-block:: python

       >>> import indicoio
       >>> import numpy as np
       >>> text = 'Monday: Delightful with mostly sunny skies. Highs in the low 70s.'
       >>> keywords = indicoio.keywords(text, top_n=3)
       >>> print "The keywords are: "+str(keywords.keys())
       u'The keywords are ['delightful', 'highs', 'skies']

    :param text: The text to be analyzed.
    :type text: str or unicode
    :rtype: Dictionary of feature score pairs
    """
    url_params = {'batch': batch, 'api_key': api_key}
    return api_handler(text, cloud=cloud, api="keywords", url_params=url_params, **kwargs)



from indicoio.utils.api import api_handler

def twitter_engagement(text, cloud=None, batch=False, api_key=None, version=None, **kwargs):
    """
    Given input text, returns an engagment score between 0 and 1

    Example usage:

    .. code-block:: python

       >>> import indicoio
       >>> import numpy as np
       >>> text = 'Monday: Delightful with mostly sunny skies. Highs in the low 70s.'
       >>> engagement = indicoio.twitter_engagement(text)

    :param text: The text to be analyzed.
    :type text: str or unicode
    :rtype: Float of engagement between 0 and 1
    """
    url_params = {"batch": batch, "api_key": api_key, "version": version}
    return api_handler(text, cloud=cloud, api="twitterengagement", url_params=url_params, **kwargs)



"""
Handles making requests to the IndicoApi Server
"""

import json, requests

from indicoio.utils.errors import IndicoError
from indicoio import JSON_HEADERS
from indicoio import config

def api_handler(arg, cloud, api, url_params=None, **kwargs):
    """
    Sends finalized request data to ML server and receives response.
    """

    if type(arg) == bytes:
        arg = arg.decode('utf-8')
    if type(arg) == list:
        arg = [a.decode('utf-8') if type(arg) == bytes else a for a in arg]
    data = {'data': arg}
    data.update(**kwargs)
    json_data = json.dumps(data)
    cloud = cloud or config.cloud
    host = "%s.indico.domains" % cloud if cloud else config.PUBLIC_API_HOST

    url = create_url(host, api, dict(kwargs, **url_params))
    response = requests.post(url, data=json_data, headers=JSON_HEADERS)

    if response.status_code == 503 and cloud != None:
        raise IndicoError("Private cloud '%s' does not include api '%s'" % (cloud, api))

    json_results = response.json()
    results = json_results.get('results', False)
    if results is False:
        error = json_results.get('error')
        raise IndicoError(error)
    return results


def create_url(host, api, url_params):
    api_key = url_params.get("api_key") or config.api_key
    is_batch = url_params.get("batch")
    apis = url_params.get("apis")
    version = url_params.get("version") or url_params.get("v")

    host_url_seg = config.url_protocol + "//%s" % host
    api_url_seg = "/%s" % api
    batch_url_seg = "/batch" if is_batch else ""
    key_url_seg = "?key=%s" % api_key
    multi_url_seg = "&apis=%s" % ",".join(apis) if apis else ""
    version_seg = ("&version=%s" % str(version)) if version else ""

    return host_url_seg + api_url_seg + batch_url_seg + key_url_seg \
            + multi_url_seg + version_seg



from functools import wraps, partial
import warnings

Version, version, __version__, VERSION = ('0.9.3',) * 4

JSON_HEADERS = {
    'Content-type': 'application/json',
    'Accept': 'application/json',
    'client-lib': 'python',
    'version-number': VERSION
}

from indicoio.text.twitter_engagement import twitter_engagement
from indicoio.text.sentiment import political, posneg, sentiment_hq
from indicoio.text.sentiment import posneg as sentiment
from indicoio.text.lang import language
from indicoio.text.tagging import text_tags
from indicoio.text.keywords import keywords
from indicoio.text.ner import named_entities
from indicoio.images.fer import fer
from indicoio.images.features import facial_features, image_features
from indicoio.images.faciallocalization import facial_localization
from indicoio.images.recognition import image_recognition
from indicoio.images.filtering import content_filtering
from indicoio.utils.multi import analyze_image, analyze_text, intersections

from indicoio.config import API_NAMES

def deprecation_decorator(f, api):
    @wraps(f)
    def wrapper(*args, **kwargs):
        warnings.warn(
            "'batch_" + api + "' will be deprecated in the next major update.  Please call '" + api + "' instead with the same arguments.",
            DeprecationWarning
        )
        return f(*args, **kwargs)
    return wrapper

def detect_batch_decorator(f):
    @wraps(f)
    def wrapper(*args, **kwargs):
        if isinstance(args[0], list):
            kwargs['batch'] = True
        return f(*args, **kwargs)
    return wrapper

apis = dict((api, globals().get(api)) for api in API_NAMES)

for api in apis:
    globals()[api] = detect_batch_decorator(apis[api])
    globals()['batch_' + api] = partial(deprecation_decorator(apis[api], api), batch=True)
